<?php

declare(strict_types=1);

namespace mirzaev\huesos\controllers\api\acquirings;

// Files of the project
use mirzaev\huesos\controllers\core,
	mirzaev\huesos\models\order,
	mirzaev\huesos\models\cart,
	mirzaev\huesos\models\product,
	mirzaev\huesos\models\telegram,
	mirzaev\huesos\models\acquirings\robokassa as model,
	mirzaev\huesos\models\enumerations\currency,
	mirzaev\huesos\models\enumerations\language;

// Framework for PHP
use mirzaev\minimal\http\enumerations\content,
	mirzaev\minimal\http\enumerations\status;

// Framework for ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Framework for Telegram
use Zanzara\Zanzara as zanzara,
	Zanzara\Context as context,
	Zanzara\Config as config,
	Zanzara\Telegram\Type\Message as message;

// Event manager for PHP
use React\EventLoop\Loop as loop;

// Built-in libraries
use DateTime as datetime,
	Exception as exception;

/**
 * Controller of robokassa
 *
 * @package mirzaev\huesos\controllers\api\acquirings
 *
 * @param array $errors Registry of errors
 *
 * @method null result() 
 * @method null success() 
 * @method null fail() 
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class robokassa extends core
{
	/**
	 * Errors
	 *
	 * @var array $errors Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => [],
		'order' => [],
		'robokassa' => []
	];

	/**
	 * Result
	 *
	 * @param null
	 */
	public function result(
		?string $OutSum = null,
		?string $InvId = null,
		?string $SignatureValue = null,
		?string $PaymentMethod = null,
		?string $IncSum = null,
		?string $IncCurrLabel = null,
		?string $IsTest = null,
		?string $EMail = null,
		?string $Fee = null,
		string ...$other
	): null {
		// Searching for the order
		$order = order::_read(
			filter: 'd._key == @_key && d.paid != true',
			sort: 'd.created DESC, d._key DESC',
			amount: 1,
			parameters: ['_key' => $InvId],
			errors: $this->errors['robokassa']
		);

		if ($order instanceof order) {
			// Initialized the order

			if (model::result(identifier: (int) $InvId, cost: $OutSum, hash: $SignatureValue, test: (bool) $IsTest, errors: $this->errors['robokassa'])) {
				// Verified the payment 

				// Initializing the cart
				$cart = $order->cart();

				if ($cart instanceof cart) {
					// Initialized the cart

					if ((float) ($cart->cost() + ($cart->buffer['delivery']['cost'] ?? 0)) === (float) $OutSum) {
						// Full payment received 

						// Writing that the order being been paid
						$order->paid = true;
						/* $order->term = null; */

						if (document::update($order->__document(), errors: $this->errors['order'])) {
							// Writed into ArangoDB

							// Initializing identifier of the order
							$identifier = $order->__document()->getKey();

							// Initializing the customer account
							$customer = $order->account();

							$config = new config();
							$config->setParseMode(config::PARSE_MODE_MARKDOWN);
							$config->useReactFileSystem(true);
							/* $config->setLoop(loop::get()); */

							$robot = new zanzara(TELEGRAM_KEY, $config);

							// Sending the message
							$robot->getTelegram()->sendMessage("✅ *Заказ \#$identifier оплачен*", ['chat_id' => $customer->identifier])
								->then(function ($message) use ($robot, $cart, $order, $identifier, $customer) {
									// Sended the message

									// Initializing the chats for sending registry
									$chats = [];

									foreach ($this->settings->chats as $chat) {
										// Iteration over chats

										if ($chat['orders']) {
											// Authorized to receive orders

											// Writing into the chats for sending registry
											$chats[] = $chat['id'];
										}
									}

									if (count($chats) > 0) {
										// Initialized chats

										// Declaring the formatted list of products for message
										$list = '';

										// Declaring total cost of products
										$cost = $cart->cost($list);

										// Escaping the formatted list of products for the message
										$list = telegram::unmarkdown($list);

										// Initializing currency symbol
										$symbol = ($account->currency ?? currency::rub)->symbol();

										// Declaring delivery texts
										$delivery_cost = $delivery_days = $delivery_address = '';

										if (!empty($cart->buffer['delivery'])) {
											// Initialized delibery data

											// Initializing delivery cost for message
											$delivery_cost = telegram::unmarkdown((string) $cart->buffer['delivery']['cost']);

											// Initializing delivery days for message
											$delivery_days = telegram::unmarkdown((string) $cart->buffer['delivery']['days']);

											// Initializing delivery address for message
											$delivery_address = telegram::unmarkdown($cart->buffer['delivery']['location']['name'] . ', ' . $cart->buffer['delivery']['street']);
										}

										// Initializing receiver domain for the message
										$domain = telegram::unmarkdown($customer->domain);

										// Initializing receiver SIM for the message
										$sim = telegram::unmarkdown((string) $customer->receiver['sim']);

										// Initializing receiver name for the message
										$name = telegram::unmarkdown($customer->receiver['name']);

										// Initializing receiver address for the message
										$address = telegram::unmarkdown($customer->receiver['address']);

										// Initializing the message cost part
										$part_cost = "*Стоимость:* $cost$symbol";
										if (!empty($delivery_cost)) $part_cost .= " \+ $delivery_cost$symbol";
										if (!empty($delivery_days)) $part_cost .= " \($delivery_days дней\)";

										// Initializing the message delivery part
										$part_delivery = '';
										if (!empty($delivery_address)) $part_delivery .= "*Адрес доставки:* $delivery_address";
										if (!empty($part_delivery)) $part_delivery = "\n$part_delivery";

										// Initializing the message receiver part
										$part_receiver = "*Получатель:* @$domain";
										if (!empty($sim)) $part_receiver .= " $sim";
										if (!empty($name)) $part_receiver .= "\n*$name*";
										if (!empty($address)) $part_receiver .= "\n\n*Адрес:* $address";

										// Initializing the message commentary part
										$part_commentary = '';
										if (!empty($order->commentary)) $part_commentary .= "\n\n*Комментарий:* " . $order->commentary;

										// Sending messages
										$robot->getTelegram()->sendBulkMessage(
											$chats,
											<<<TXT
											📦 *Заказ* \#$identifier
	
											$list
											$part_cost$part_delivery	

											$part_receiver$part_commentary
											TXT,
											[
												'reply_markup' => [
													'inline_keyboard' => [
														[
															['text' => '✉️ Чат с покупателем', 'url' => 'https://t.me/' . $customer->domain]
														]
													],
													'disable_notification' => true
												]
											]
										);
									}
								});

							// Sending response
							$this->response
								->start()
								->clean()
								->sse()
								->write("OK$identifier")
								->validate($this->request)
								?->body()
								->end();
						} else throw new exception('Failed to update the order');
					}
				}
			}
		}

		// Exit (success/fail)
		return null;
	}

	/**
	 * Success
	 *
	 * @return null
	 */
	public function success(
		?string $OutSum = null,
		?string $InvId = null,
		?string $SignatureValue = null,
		?string $IsTest = null,
		?string $Culture = null,
		string ...$other
	): null {
		// Searching for the order
		$order = order::_read(
			filter: 'd._key == @_key',
			sort: 'd.created DESC, d._key DESC',
			amount: 1,
			parameters: ['_key' => $InvId],
			errors: $this->errors['robokassa']
		);

		if ($order instanceof order) {
			// Initialized the order

			if (model::success(identifier: (int) $InvId, cost: $OutSum, hash: $SignatureValue, test: (bool) $IsTest, errors: $this->errors['robokassa'])) {
				// Verified the payment 

				if ($IsTest == 1) {
					// Enabled the test mode

					// Initializing a paragraph abount the test mode
					$this->view->test = $this->language === language::ru ? 'Тестовый режим' : 'Test mode';
				}

				// Render page
				$page = $this->view->render('api/acquirings/robokassa/success.html', [
					'h2' => $this->language === language::ru ? 'Заказ #' . $order->getKey() . ' оплачен' : 'Order #' . $order->getKey() . ' paid',
					'identifier' => $order->getKey(),
					'closing' => [
						'title' => $this->language === language::ru ? 'Закрытие окна' : 'Window closing',
						'iterator' => 30
					]
				]);

				// Sending response
				$this->response
					->start()
					->clean()
					->sse()
					->write($page)
					->validate($this->request)
					?->body()
					->end();

				// Deinitializing rendered page
				unset($page);
			}
		}

		// Exit (success/fail)
		return null;
	}

	/**
	 * Fail
	 *
	 * @return null
	 */
	public function fail(
		?string $OutSum = null,
		?string $InvId = null,
		?string $IsTest = null,
		?string $Culture = null,
		string ...$other
	): null {
		// Searching for the order
		$order = order::_read(
			filter: 'd._key == @_key',
			sort: 'd.created DESC, d._key DESC',
			amount: 1,
			parameters: ['_key' => $InvId],
			errors: $this->errors['robokassa']
		);

		if ($order instanceof order) {
			// Initialized the order

			if ($IsTest == 1) {
				// Enabled the test mode

				// Initializing a paragraph abount the test mode
				$this->view->test = $this->language === language::ru ? 'Тестовый режим' : 'Test mode';
			}

			// Render page
			$page = $this->view->render('api/acquirings/robokassa/fail.html', [
				'h2' => $this->language === language::ru ? 'Заказ #' . $order->getKey() . ' не оплачен' : 'Order #' . $order->getKey() . ' not paid',
				'identifier' => $order->getKey(),
				'closing' => [
					'title' => $this->language === language::ru ? 'Закрытие окна' : 'Window closing',
					'iterator' => 30
				]
			]);

			// Sending response
			$this->response
				->start()
				->clean()
				->sse()
				->write($page)
				->validate($this->request)
				?->body()
				->end();

			// Deinitializing rendered page
			unset($page);
		}

		// Exit (success/fail)
		return null;
	}
}
