<?php

declare(strict_types=1);

namespace mirzaev\huesos\models;

// Files of the project
use mirzaev\huesos\models\core,
	mirzaev\huesos\models\reservation,
	mirzaev\huesos\models\traits\buffer,
	mirzaev\huesos\models\traits\document as document_trait,
	mirzaev\huesos\models\interfaces\document as document_interface,
	mirzaev\huesos\models\interfaces\collection as collection_interface,
	mirzaev\huesos\models\enumerations\language,
	mirzaev\huesos\models\enumerations\currency;

// Framework for ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Library for ArangoDB
use ArangoDBClient\Document as _document;

// Built-in libraries
use DateTime as datetime,
	Exception as exception;

/**
 * Model of cart
 *
 * @uses reservation
 * @package mirzaev\huesos\models
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class cart extends core implements document_interface, collection_interface
{
	use document_trait, buffer {
		buffer::write as buffer_write;
	}

	/**
	 * Name of the collection in ArangoDB
	 */
	final public const string COLLECTION = 'cart';

	/**
	 * Search for all products
	 *
	 * Search for all products in the cart
	 *
	 * @param language|null $language Language
	 * @param currency|null $currency Currency
	 * @param array &$errors Registry of errors
	 *
	 * @return array|null Array with implementing objects of documents from ArangoDB, if found
	 */
	public function products(
		?language $language = language::en,
		?currency $currency = currency::usd,
		array &$errors = []
	): ?array {
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				if (collection::initialize(reservation::COLLECTION, reservation::TYPE, errors: $errors)) {
					if (collection::initialize(product::COLLECTION, product::TYPE, errors: $errors)) {
						// Initialized collections

						// Search for all products in the cart
						$result = @collection::execute(
							<<<AQL
								FOR v IN 1..1 INBOUND @cart GRAPH @graph
								FILTER IS_SAME_COLLECTION(@collection, v._id)
								COLLECT d = v WITH COUNT INTO amount
								RETURN {
									[d._id]: {
										amount,
										document: MERGE(d, {
											name: d.name.@language, 
											description: d.description.@language, 
											compatibility: d.compatibility.@language, 
											brand: d.brand.@language, 
											cost: d.cost.@currency
										})		
									}
								}
							AQL,
							[
								'graph' => 'catalog',
								'cart' => $this->getId(),
								'collection' => product::COLLECTION,
								'language' => $language->name,
								'currency' => $currency->name
							],
							flat: true,
							errors: $errors
						);

						/*
						* МеНЯ ЭТО РАЗДРАЖАЕТ
						*/

						$products = [];

						foreach ($result ?? [] as $raw) {
							foreach ($raw as $key => $value) {
								$products[$key] = $value;
							}
						}

						// Exit (success)
						return isset($products['amount']) ? [$products['document']['_id'] => $products] : $products;
					} else throw new exception('Failed to initialize ' . product::TYPE->name . ' collection: ' . product::COLLECTION);
				} else throw new exception('Failed to initialize ' . reservation::TYPE->name . ' collection: ' . reservation::COLLECTION);
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Search for summary of all products
	 *
	 * Search for summary of all products in the cart
	 *
	 * @param currency|null $currency Currency
	 * @param array &$errors Registry of errors
	 *
	 * @return array|null Array with implementing objects of documents from ArangoDB, if found
	 */
	public function summary(
		?currency $currency = currency::usd,
		array &$errors = []
	): ?array {
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				if (collection::initialize(reservation::COLLECTION, reservation::TYPE, errors: $errors)) {
					if (collection::initialize(product::COLLECTION, product::TYPE, errors: $errors)) {
						// Initialized collections

						// Search for all products in the cart
						$result = @collection::execute(
							<<<AQL
						FOR v IN 1..1 INBOUND @cart GRAPH @graph
							FILTER IS_SAME_COLLECTION(@collection, v._id)
							COLLECT AGGREGATE amount = COUNT(v), cost = SUM(v.cost.@currency)
							RETURN { amount, cost }
						AQL,
							[
								'graph' => 'catalog',
								'cart' => $this->getId(),
								'collection' => product::COLLECTION,
								'currency' => $currency->name
							],
							flat: true,
							errors: $errors
						);

						// Exit (success)
						return $result;
					} else throw new exception('Failed to initialize ' . product::TYPE->name . ' collection: ' . product::COLLECTION);
				} else throw new exception('Failed to initialize ' . reservation::TYPE->name . ' collection: ' . reservation::COLLECTION);
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Count
	 *
	 * Count of the product in the cart
	 *
	 * @param product $product The product
	 * @param int $limit Limit for counting
	 * @param array &$errors Registry of errors
	 *
	 * @return int|null Amount of the product in the cart, if counted
	 */
	public function count(product $product, int $limit = 100, array &$errors = []): ?int
	{
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				if (collection::initialize(reservation::COLLECTION, reservation::TYPE, errors: $errors)) {
					if (collection::initialize(product::COLLECTION, product::TYPE, errors: $errors)) {
						// Initialized collections

						// Search for the products in the cart and count them
						return (int) collection::execute(
							<<<AQL
							FOR v IN 1..1 INBOUND @cart GRAPH @graph
								FILTER IS_SAME_COLLECTION(@collection, v._id) && v._id == @product
								LIMIT @limit
								COLLECT WITH COUNT INTO length
								RETURN length
							AQL,
							[
								'graph' => 'catalog',
								'cart' => $this->getId(),
								'collection' => product::COLLECTION,
								'product' => $product->getId(),
								'limit' => $limit
							],
							errors: $errors
						);
					} else throw new exception('Failed to initialize ' . product::TYPE->name . ' collection: ' . product::COLLECTION);
				} else throw new exception('Failed to initialize ' . reservation::TYPE->name . ' collection: ' . reservation::COLLECTION);
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}

	/*
	 * Write
	 *
	 * Write the product into the cart
	 *
	 * @param product $product The product
	 * @param int $amount Amount of writings
	 * @param array &$errors Registry of errors
	 *
	 * @return void
	 */
	public function write(product $product, int $amount = 1, array &$errors = []): void
	{
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				if (collection::initialize(reservation::COLLECTION, reservation::TYPE, errors: $errors)) {
					if (collection::initialize(product::COLLECTION, product::TYPE, errors: $errors)) {
						// Initialized collections

						// Writing the product to the cart
						collection::execute(
							<<<AQL
							FOR i IN 1..@amount
								INSERT { _from: @product, _to: @cart, created: @created, updated: @created, active: true } INTO @@edge
							AQL,
							[
								'cart' => $this->getId(),
								'product' => $product->getId(),
								'@edge' => reservation::COLLECTION,
								'amount' => $amount,
								'created' => time()
							],
							errors: $errors
						);
					} else throw new exception('Failed to initialize ' . product::TYPE->name . ' collection: ' . product::COLLECTION);
				} else throw new exception('Failed to initialize ' . reservation::TYPE->name . ' collection: ' . reservation::COLLECTION);
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
	}

	/*
	 * Delete
	 *
	 * Delete the product from the cart
	 *
	 * @param product $product The product
	 * @param int $amount Amount of deletings
	 * @param array &$errors Registry of errors
	 *
	 * @return void
	 */
	public function delete(product $product, int $amount = 1, array &$errors = []): void
	{
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				if (collection::initialize(reservation::COLLECTION, reservation::TYPE, errors: $errors)) {
					if (collection::initialize(product::COLLECTION, product::TYPE, errors: $errors)) {
						// Initialized collections

						// Deleting the product from the cart
						collection::execute(
							<<<AQL
							FOR v, e IN 1..1 INBOUND @cart GRAPH @graph
								FILTER IS_SAME_COLLECTION(@collection, v._id) && v._id == @product
								LIMIT @amount
								REMOVE e._key IN @@reservation
							AQL,
							[
								'graph' => 'catalog',
								'cart' => $this->getId(),
								'collection' => product::COLLECTION,
								'product' => $product->getId(),
								'@reservation' => reservation::COLLECTION,
								'amount' => $amount
							],
							errors: $errors
						);
					} else throw new exception('Failed to initialize ' . product::TYPE->name . ' collection: ' . product::COLLECTION);
				} else throw new exception('Failed to initialize ' . reservation::TYPE->name . ' collection: ' . reservation::COLLECTION);
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
	}

	/*
	 * Share
	 *
	 * Generate hash for sharing the cart
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return string|false Hash for sharing, if generated and writed to ArangoDB
	 */
	public function share(array &$errors = []): string|false
	{
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				// Initialized collection

				// Generating hash and writing to the cart implement instance
				$this->share = sodium_bin2hex(sodium_crypto_generichash($this->getId()));

				if (document::update($this->__document(), errors: $errors)) {
					// Writed to ArangoDB

					// Exit (success)
					return $this->share;
				} else throw new exception('Failed to write confirmed cart to ArangoDB');
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return false;
	}

	/*
	 * Unshare
	 *
	 * Deleting hash for sharing the cart
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return bool Is the cart unshared?
	 */
	public function unshare(array &$errors = []): bool
	{
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				// Initialized collection

				// Deleting hash and writing to the cart implement instance
				$this->share = null;

				if (document::update($this->__document(), errors: $errors)) {
					// Writed to ArangoDB

					// Exit (success)
					return true;
				} else throw new exception('Failed to write confirmed cart to ArangoDB');
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return false;
	}

	/**
	 * Account
	 *
	 * Search for the connected account
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return account|null The connected account, if found
	 */
	public function account(array &$errors = []): ?account
	{
		try {
			if ($result = collection::execute(
				<<<'AQL'
				FOR v IN 1..1 OUTBOUND @cart GRAPH users
					FILTER PARSE_IDENTIFIER(v._id).collection == @account
					LIMIT 1
					return v
				AQL,
				[
					'cart' => $this->document->getId(),
					'account' => account::COLLECTION
				],
				errors: $errors
			)) {
				// Found the instance of the account connected to the account

				// Initializing the object
				$account = new account;

				if (method_exists($account, '__document')) {
					// Object can implement a document from ArangoDB

					// Implementinf parameters
					if (isset($result->language)) $result->language = language::{$result->language};
					if (isset($result->currency)) $result->currency = currency::{$result->currency};

					// Writing the instance of the account document from ArangoDB to the implement object
					$account->__document($result);

					// Exit (success)
					return $account;
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
		// Exit (fail)
		return null;
	}

	/**
	 * Cost
	 *
	 * Generate the cart products cost
	 *
	 * @param string &$list List of the calculated products
	 * @param array &$errors Registry of errors
	 *
	 * @return float|int|null The cart products cost, if generated
	 */
	public function cost(string &$list = '', array &$errors = []): float|int|null
	{
		try {
			// Initializing the account
			$account = $this->account($errors);

			if ($account instanceof account) {
				// Initialized the account

				// Initializing products in the cart
				$products = $this->products(language: $account->language ?? language::ru, currency: $account->currency ?? currency::rub);

				if (!empty($products)) {
					// Initialized products in the cart

					// Declaring total cost of products
					$cost = 0;

					// Initializing iterator of rows
					$row = 0;

					foreach ($products as $product) {
						// Iterating over products

						// Generating formatted list of products for message
						$list .= ++$row . '. ' . $product['document']['name'] . ' (' . $product['amount'] . 'шт)' . "\n";

						// Generating total cost of products
						$cost += $product['document']['cost'] * $product['amount'];
					}

					// Exit (success)
					return $cost;
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
		// Exit (fail)
		return null;
	}

	/*
	 * Order
	 *
	 * Create the order and connect to the cart and to the account
	 * Make the cart ordered (the account will create a new cart)
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return order|null The order, if created
	 *
	 * @todo Handling errors
	 */
	public function order(array &$errors = []): ?order
	{
		try {
			if (collection::initialize(static::COLLECTION, static::TYPE, errors: $errors)) {
				if (collection::initialize(order::COLLECTION, order::TYPE, errors: $errors)) {
					if (collection::initialize(reservation::COLLECTION, reservation::TYPE, errors: $errors)) {
						if (collection::initialize(account::COLLECTION, account::TYPE, errors: $errors)) {
							// Initialized collections

							// Searching for the order
							$existed = collection::execute(
								<<<'AQL'
								FOR v IN 1..1 OUTBOUND @cart GRAPH users
									FILTER PARSE_IDENTIFIER(v._id).collection == @order
									LIMIT 1
									return v
								AQL,
								[
									'cart' => $this->document->getId(),
									'order' => order::COLLECTION
								],
								errors: $errors
							);

							if ($existed) {
								// The order has already been created

								// Initializing the object
								$order = new order;

								if (method_exists($order, '__document')) {
									// Object can implement a document from ArangoDB

									// Writing the instance of the order document from ArangoDB to the implement object
									$order->__document($existed);

									// Exit (success)
									return $order;
								}
							} else {
								// The order has not been created

								// Initializing a new order
								$_id = document::write(
									order::COLLECTION,
									[
										'active' => true,
										/* 'term' => (int) new datetime('+15 minutes')->ormat('U') */
									]
								);

								if ($result = collection::execute(
									<<<'AQL'
									FOR d IN @@collection
										FILTER d._id == @_id && d.active == true
										RETURN d
									AQL,
									[
										'@collection' => order::COLLECTION,
										'_id' => $_id
									],
									errors: $errors
								)) {
									// Found the instance of just created the new order

									// Writing the ordered status for the cart
									$this->document->ordered = true;

									if (document::update($this->__document(), errors: $errors)) {
										// Writed into ArangoDB

										// Initializing the object
										$order = new order;

										if (method_exists($order, '__document')) {
											// Object can implement a document from ArangoDB

											// Writing the instance of the order document from ArangoDB to the implement object
											$order->__document($result);

											// Connecting the cart to the order
											$connected = $order->connect($this, $errors);

											if ($connected) {
												// Connected the cart with the order

												if ($result = collection::execute(
													<<<'AQL'
													FOR v IN 1..1 OUTBOUND @cart GRAPH users
														FILTER PARSE_IDENTIFIER(v._id).collection == @account
														LIMIT 1
														return v
													AQL,
													[
														'cart' => $this->document->getId(),
														'account' => account::COLLECTION
													],
													errors: $errors
												)) {
													// Found the instance of the account connected to the cart

													// Initializing the object
													$account = new account;

													if (method_exists($account, '__document')) {
														// Object can implement a document from ArangoDB

														// Writing the instance of the account document from ArangoDB to the implement object
														$account->__document($result);

														// Connecting the account with the order 
														$connected = $account->connect($order, $errors);

														if ($connected) {
															// Connected the account with the order

															// Exit (success)
															return $order;
														}
													}
												}
											}
										} else throw new exception('Class ' . order::class . ' does not implement a document from ArangoDB');
									} else throw new exception('Failed to write the ordered status for the cart');
								} else throw new exception('Failed to create or find just created ' . static::class);
							}
						} else throw new exception('Failed to initialize ' . account::TYPE->name . ' collection: ' . account::COLLECTION);
					} else throw new exception('Failed to initialize ' . reservation::TYPE->name . ' collection: ' . reservation::COLLECTION);
				} else throw new exception('Failed to initialize ' . order::TYPE->name . ' collection: ' . order::COLLECTION);
			} else throw new exception('Failed to initialize ' . static::TYPE->name . ' collection: ' . static::COLLECTION);
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}
}
