<?php

declare(strict_types=1);

namespace mirzaev\huesos\models;

// Files of the project
use mirzaev\huesos\models\core,
	mirzaev\huesos\controllers\core as controller,
	mirzaev\huesos\models\catalog,
	mirzaev\huesos\models\suspension,
	mirzaev\huesos\models\account,
	mirzaev\huesos\models\acquirings\robokassa,
	mirzaev\huesos\models\enumerations\language,
	mirzaev\huesos\models\enumerations\currency,
	mirzaev\huesos\models\enumerations\destination;

// Framework for Telegram
use Zanzara\Zanzara,
	Zanzara\Context as context,
	Zanzara\Telegram\Type\Input\InputFile as input,
	Zanzara\Telegram\Type\File\Document as telegram_document,
	Zanzara\Middleware\MiddlewareNode as Node,
	Zanzara\Telegram\Type\User as user;

// Framework for asynchronous PHP
use function React\Async\await;

// Framework for ArangoDB
use mirzaev\arangodb\document;

// Built-in libraries
use DateTime as datetime;

/**
 * Model of chat (telegram)
 *
 * @package mirzaev\huesos\models
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class telegram extends core
{
	/**
	 * Экранирование символов для Markdown
	 *
	 * @param string $text Текст для экранирования
	 * @param array $exception Символы которые будут исключены из списка для экранирования
	 *
	 * @return string Экранированный текст
	 */
	public static function unmarkdown(string $text, array $exceptions = []): string
	{
		// Инициализация реестра символом для конвертации
		$from = array_diff(
			[
				'#',
				'*',
				'_',
				'=',
				'.',
				'[',
				']',
				'(',
				')',
				'-',
				'>',
				'<',
				'!',
				'`'
			],
			$exceptions
		);

		// Инициализация реестра целей для конвертации
		$to = [];
		foreach ($from as $symbol) $to[] = "\\$symbol";

		// Конвертация и выход (успех)
		return str_replace($from, $to,	$text);
	}

	/**
	 * Инициализация запчасти
	 *
	 * Проверяет существование запчасти
	 *
	 * @param string $spare Запчасть
	 *
	 * @return string|bool Запчасть, если найдена, иначе false
	 */
	public static function spares(string $spare): string|bool
	{
		// Поиск запчастей и выход (успех)
		return match (mb_strtolower($spare)) {
			'цевьё' => 'Цевьё',
			default => false
		};
	}

	/**
	 * Главное меню
	 *
	 * Команда: /start
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function menu(context $context): void
	{
		// Declaring keyboard buttond
		$keyboard = [];

		/* if ($context->get('account')) {
			// Authenticated */

		// Writing the catalog button
		$keyboard[] = [
			[
				'text' => '🛒 Открыть магазин',
				'web_app' => ['url' => 'https://telegram.arming.ru/']
			]
		];
		/* } */

		$keyboard = array_merge(
			$keyboard,
			[[
				['text' => '🏛️ О компании', 'callback_data' => 'company'],
				['text' => '💬 Контакты', 'callback_data' => 'contacts']
			]],
			[[
				['text' => '🎯 Сообщество', 'callback_data' => 'community']
			]]
		);


		if ($context->get('account')?->access['settings']) {
			// Authorized for settings

			// Writing the settings button
			$keyboard[] = [['text' => '⚙️  Настройки', 'callback_data' => 'settings']];
		}

		if (defined('GREETING_VIDEO') && file_exists(GREETING_VIDEO)) {
			// Video message

			// Sending the video
			await($context->sendVideo(new input(GREETING_VIDEO), ['disable_notification' => true]));
		}

		// Sending the message
		$context->sendMessage(
			static::unmarkdown(<<<TXT
			Добро пожаловать в ARМING! 🔥

			Здесь ты найдешь тактическое снаряжение, оружейный тюнинг, кемпинг.

			⭐️ Заказывай быстро и удобно прямо в Telegram.
			⭐️ Только проверенные товары без переплат.
			⭐️ Качество без компромиссов.

			🚚 Доставка бесплатно по всей России!			
			TXT),
			[
				'reply_markup' => [
					'inline_keyboard' => $keyboard,
					'resize_keyboard' => true
				],
				'disable_notification' => true
			]
		);
	}

	/**
	 * Начало работы с чат-роботом
	 *
	 * Команда: /start
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function start(context $context): void
	{
		// Главное меню
		static::menu($context);
	}

	/**
	 * Контакты
	 *
	 * Команда: /contacts
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function contacts(context $context): void
	{
		// Отправка сообщения
		$context->sendMessage(static::unmarkdown(<<<TXT
		Здесь придумать текст для раздела "Контакты"
		TXT), [
			'reply_markup' => [
				'inline_keyboard' => [
					[
						['text' => '⚡ Связь с менеджером', 'url' => 'https://t.me/iarming'],
					],
					[
						['text' => '📨 Почта', 'callback_data' => 'mail']
					],
					[
						['text' => '🪖 Сайт', 'url' => 'https://arming.ru'],
						['text' => '🛒 Wildberries', 'url' => 'https://www.wildberries.ru/seller/137386'],
						['text' => '🛒 Ozon', 'url' => 'https://www.ozon.ru/seller/arming-1086587/products/?miniapp=seller_1086587'],
					]
				]
			],
			'link_preview_options' => [
				'is_disabled' => true
			],
			'disable_notification' => true
		]);
	}

	/**
	 * Почта
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function _mail(context $context): void
	{
		// Отправка сообщения
		$context->sendMessage(static::unmarkdown(<<<TXT
		[info@arming.ru](mailto::info@arming.ru)
		TXT, ['[', ']', '(', ')']), [
			'link_preview_options' => [
				'is_disabled' => true
			],
			'disable_notification' => true
		]);
	}

	/**
	 * Компания
	 *
	 * Команда: /company
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function company(context $context): void
	{
		// Отправка сообщения
		$context->sendMessage(
			static::unmarkdown(<<<TXT
			Здесь придумать текст для раздела "Компания"
			TXT),
			[
				'reply_markup' => [
					'inline_keyboard' => [
						[
							['text' => '📄 Публичная оферта', 'web_app' => ['url' => 'https://telegram.arming.ru/offer']],
						]
					]
				],
				'link_preview_options' => [
					'is_disabled' => true
				]
			]
		);
	}

	/**
	 * Сообщество
	 *
	 * Команда: /community
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function community(context $context): void
	{
		// Отправка сообщения
		$context->sendMessage(static::unmarkdown(<<<TXT
		Здесь придумать текст для раздела "Сообщество"
		TXT), [
			'reply_markup' => [
				'inline_keyboard' => [
					[
						['text' => '💬 Основной чат', 'url' => 'https://t.me/arming_zone'],
					]
				]
			],
			'link_preview_options' => [
				'is_disabled' => true
			],
			'disable_notification' => true
		]);
	}

	/**
	 * Настройки (доступ только авторизованным)
	 *
	 * Команда: /settings
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function settings(context $context): void
	{
		if ($context->get('account')?->access['settings']) {
			// Авторизован доступ к настройкам

			// Отправка сообщения
			$context->sendMessage(
				static::unmarkdown(<<<TXT
				Панель управления чат-роботом ARMING
				TXT),
				[
					'reply_markup' => [
						'inline_keyboard' => [
							[
								['text' => '📦 Импорт товаров', 'callback_data' => 'import_request'],
							]
						]
					],
					'link_preview_options' => [
						'is_disabled' => true
					],
					'disable_notification' => true
				]
			);
		} else {
			// Не авторизован доступ к настройкам

			// Отправка сообщения
			$context->sendMessage('⛔ *Нет доступа*');
		}
	}

	/**
	 * Запросить файл для импорта товаров (доступ только авторизованным)
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function import_request(context $context): void
	{
		if ($context->get('account')?->access['settings']) {
			// Авторизован доступ к настройкам

			// Отправка сообщения
			$context->sendMessage(static::unmarkdown('Отправьте документ в формате xlsx со списком товаров'))
				->then(function ($message) use ($context) {
					// Отправка файла
					$context->sendDocument(new input(CATALOG_EXAMPLE), ['disable_notification' => true]);

					// Импорт файла
					$context->nextStep([static::class, 'import'], true);
				});
		} else {
			// Не авторизован доступ к настройкам

			// Отправка сообщения
			$context->sendMessage('⛔ *Нет доступа*');
		}
	}

	/**
	 * Импорт товаров (доступ только авторизованным)
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function import(context $context): void
	{
		if (($account = $context->get('account'))?->access['settings']) {
			// Авторизован доступ к настройкам

			// Инициализация документа
			$document = $context->getMessage()?->getDocument();

			if ($document instanceof telegram_document) {
				// Инициализирован документ

				// Инициализация файла
				$context->getFile($document->getFileId())->then(function ($file) use ($context, $document, $account) {

					if ($file->getFileSize() <= 50000000) {
						// Не превышает 50 мегабайт (50 000 000 байт) размер файла

						if (pathinfo(parse_url($file->getFilePath())['path'], PATHINFO_EXTENSION) === 'xlsx') {
							// Имеет расширение xlsx файл

							// Initializing the account imports directory in the storage
							if (!file_exists($storage = STORAGE . DIRECTORY_SEPARATOR . 'import' . DIRECTORY_SEPARATOR . $account->getKey() . DIRECTORY_SEPARATOR . time()))
								mkdir(directory: $storage, permissions: 0775, recursive: true);

							// Сохранение файла
							file_put_contents(
								$import = $storage . DIRECTORY_SEPARATOR . 'import.xlsx',
								file_get_contents('https://api.telegram.org/file/bot' . KEY . '/' . parse_url($file->getFilePath())['path'])
							);

							// Отправка сообщения
							$context->sendMessage(sprintf(
								<<<'TXT'
								🔬 *Выполняется анализ:* %s \(%s байт\)
								TXT,
								static::unmarkdown($document->getFileName()),
								static::unmarkdown((string) $file->getFileSize())
							))
								->then(function ($message) use ($context, $import) {
									// Инициализация счётчика загруженных товаров
									$categories_loaded
										= $products_loaded
										= $categories_created
										= $products_created
										= $categories_updated
										= $products_updated
										= $categories_deleted
										= $products_deleted
										= $categories_old
										= $products_old
										= $categories_new
										= $products_new
										= 0;

									// Import
									catalog::import(
										$import,
										$categories_loaded,
										$categories_created,
										$categories_updated,
										$categories_deleted,
										$categories_old,
										$categories_new,
										$products_loaded,
										$products_created,
										$products_updated,
										$products_deleted,
										$products_old,
										$products_new,
										language: $account->language ?? settings::active()?->language ?? language::en, // @todo add languages
										currency: $account->currency ?? settings::active()?->currency ?? currency::usd // @todo add currencies
									);

									// Отправка сообщения
									$context->sendMessage(<<<TXT
										🏷 *Категории*

										*Загружено:* $categories_loaded

										*Добавлено:* $categories_created
										*Обновлено:* $categories_updated
										*Удалено:* $categories_deleted

										*Было:* $categories_old
										*Стало:* $categories_new
										TXT)
										->then(function ($message) use ($context, $products_loaded, $products_created, $products_updated, $products_deleted, $products_old, $products_new) {
											$context->sendMessage(<<<TXT
												📦 *Товары*

												*Загружено:* $products_loaded

												*Добавлено:* $products_created
												*Обновлено:* $products_updated
												*Удалено:* $products_deleted

												*Было:* $products_old
												*Стало:* $products_new
												TXT)
												->then(function ($message) use ($context) {
													// Завершение диалога
													$context->endConversation();
												});
										});
								});
						} else {
							// Не имеет расширение xlsx файл

							// Отправка сообщения
							$context->sendMessage(static::unmarkdown('Файл должен иметь расширение xlsx'));
						}
					} else {
						// Превышает 50 мегабайт (50000000 байт) размер файла

						// Отправка сообщения
						$context->sendMessage(static::unmarkdown('Размер файла не должен превышать 50 мегабайт'));
					}
				});
			} else {
				// Не инициализирован документ

				// Отправка сообщения
				$context->sendMessage(static::unmarkdown('Отправьте документ в формате xlsx со списком товаров'));
			}
		} else {
			// Не авторизован доступ к настройкам

			// Отправка сообщения
			$context->sendMessage('⛔ *Нет доступа*');
		}
	}

	/**
	 * Инициализация аккаунта (middleware)
	 *
	 * @param context $context
	 * @param Node $next
	 *
	 * @return void
	 */
	public static function account(context $context, Node $next): void
	{
		// Выполнение заблокировано?
		if ($context->get('stop')) return;

		// Инициализация аккаунта Telegram
		$telegram = $context->getEffectiveUser();

		// Инициализация аккаунта
		$account = account::initialize($telegram->getId(), $telegram);

		if ($account) {
			// Инициализирован аккаунт

			if ($account->banned) {
				// Заблокирован аккаунт

				// Отправка сообщения
				$context->sendMessage('⛔ *Ты заблокирован*')
					->then(function ($message) use ($context) {
						// Завершение диалога
						$context->endConversation();
					});

				// Блокировка дальнейшего выполнения
				$context->set('stop', true);
			} else {
				// Не заблокирован аккаунт

				// Запись в буфер
				$context->set('account', $account);

				// Продолжение выполнения
				$next($context);
			}
		} else {
			// Не инициализирован аккаунт
		}
	}

	/**
	 * Инициализация статуса технических работ (middleware)
	 *
	 * @param context $context
	 * @param Node $next
	 *
	 * @return void
	 */
	public static function suspension(context $context, Node $next): void
	{
		// Выполнение заблокировано?
		if ($context->get('stop')) return;

		// Поиск технических работ
		$suspension = suspension::search();

		if ($suspension && $suspension->targets['telegram-robot']) {
			// Найдена активная приостановка

			// Инициализация аккаунта
			$account = $context->get('account');

			if ($account) {
				// Инициализирован аккаунт

				foreach ($suspension->access as $type => $status) {
					// Перебор статусов доступа

					if ($status && $account->{$type}) {
						// Авторизован аккаунт

						// Продолжение выполнения
						$next($context);

						// Выход (успех)
						return;
					}
				}
			}

			// Инициализация сообщения
			$message = "⚠️ *Работа приостановлена*\n*Оставшееся время\:* " . $suspension->message($account->language ?? settings::active()?->language ?? 'en');

			// Добавление описания причины приостановки, если найдена
			if (!empty($suspension->description))
				$message .= "\n\n" . $suspension->description[$account->language ?? settings::active()?->language ?? 'en'] ?? array_values($suspension->description)[0];

			// Отправка сообщения
			$context->sendMessage($message)
				->then(function ($message) use ($context) {
					// Завершение диалога
					$context->endConversation();
				});

			// Блокировка дальнейшего выполнения
			$context->set('stop', true);
		} else {
			// Не найдена активная приостановка

			// Продолжение выполнения
			$next($context);
		}
	}

	/**
	 * Cart attach
	 *
	 * @param context $context
	 * @param string $share Sharing hash
	 *
	 * @return void
	 */
	public static function cart_attach(context $context, string $share): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing cart
			$cart = cart::_read(
				filter: 'd.share == @share',
				sort: 'd.updated DESC, d.created DESC, d._key DESC',
				amount: 1,
				page: 1,
				parameters: ['share' => $share]
			);

			// Deinitializing unnecessary variables
			unset($share);

			if ($cart instanceof cart) {
				// Initialized the cart

				// Unsharing the cart
				$cart->unshare();

				// Connecting the cart to the account
				$edge = $account->connect($cart);

				if (!empty($edge)) {
					// Connected the cart to the account

					// Initializing products in the cart
					$products = $cart->products(language: $account->language ?? language::ru, currency: $account->currency ?? currency::rub);

					if (!empty($products)) {
						// Initialized products in the cart

						// Declaring the formatted list of products for message
						$list = '';

						// Declaring total cost of products
						$cost = $cart->cost($list);

						// Escaping the formatted list of products for the message
						$list = static::unmarkdown($list);

						// Deinitializing unnecessary variables
						unset($products, $product, $row);

						// Initializing identifier of the cart
						$identifier = $cart->getKey();

						// Initializing currency symbol
						$symbol = ($account->currency ?? currency::rub)->symbol();

						// Declaring delivery texts
						$delivery_cost = $delivery_days = $delivery_address = '';

						if (!empty($cart->buffer['delivery'])) {
							// Initialized delibery data

							// Initializing delivery cost for message
							$delivery_cost = static::unmarkdown((string) $cart->buffer['delivery']['cost']);

							// Initializing delivery days for message
							$delivery_days = static::unmarkdown((string) $cart->buffer['delivery']['days']);

							// Initializing delivery address for message
							$delivery_address = static::unmarkdown($cart->buffer['delivery']['location']['name'] . ', ' . $cart->buffer['delivery']['street']);
						}

						// Initializing the message cost part
						$part_cost = "*Стоимость:* $cost$symbol";
						if (!empty($delivery_cost)) $part_cost .= " \+ $delivery_cost$symbol";
						if (!empty($delivery_days)) $part_cost .= " \($delivery_days дней\)";

						// Initializing the message delivery part
						$part_delivery = '';
						if (!empty($delivery_address)) $part_cost .= "*Адрес доставки:* $delivery_address";
						if (!empty($part_delivery)) $part_delivery = "\n$part_delivery";

						$context->sendMessage(
							<<<TXT
							🛒 *Добавлена корзина* \#$identifier

							$list
							$part_cost$part_delivery	
							TXT,
							[
								'reply_markup' => [
									'inline_keyboard' => [
										[
											['text' => '🚚 Оформить доставку', 'callback_data' => 'cart_delivery'],
										],
									],
									'disable_notification' => true
								]
							]
						);
					}

					// Deinitializing unnecessary variables
					unset($cart, $list);
				}
			}

			// Deinitializing unnecessary variables
			unset($cart);
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Cart delivery
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function cart_delivery(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing text of the message
			$text = $context->getCallbackQuery()?->getMessage()?->getText() ?? '';

			// Searchin for the cart identifier
			preg_match('/^.*\#(\d+)$/mu', $text, $matches);

			// Initializing the cart identifier
			$identifier = $matches[1];

			if ($identifier) {
				// Initialized the cart identifier

				// Initializing cart
				$cart = cart::_read(
					filter: 'd._key == @_key',
					sort: 'd.updated DESC, d.created DESC, d._key DESC',
					amount: 1,
					page: 1,
					parameters: ['_key' => $identifier]
				);

				if ($cart instanceof cart) {
					// Initialized the cart

					// Initializin the account connected to the cart
					$connected = $cart->account();

					if ($account->getId() === $connected->getId()) {
						// The account is authorized to the cart

						// Writing to the runtime buffer
						$context->setUserDataItem('cart', $cart)
							->then(function () use ($context) {
								static::delivery_registration($context);
							});
					}
				}

				// Deinitializing unnecessary variables
				unset($cart);
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Delivery registration
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function delivery_registration(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing the settings
			$settings = settings::active();

			// Initializing requirements
			$requirements = [
				'sim' => array_search('sim', $settings->input['deliveries']['chat'], true) !== false,
				'name' => array_search('name', $settings->input['deliveries']['chat'], true) !== false,
				'destination' => array_search('destination', $settings->input['deliveries']['chat'], true) !== false,
				'address' => array_search('address', $settings->input['deliveries']['chat'], true) !== false,
				'commentary' => array_search('commentary', $settings->input['deliveries']['chat'], true) !== false,
			];

			// Ending the conversation
			$context->endConversation()
				->then(function () use ($context, $requirements, $account) {
					// Ended the conversation

					switch (true) {
						case !empty($requirements['sim']) && empty($account->receiver['sim']):
							// SIM-number

							/* // Sending the message
							$context->sendMessage(
								<<<TXT
								📞 *Пришлите SIM\-номер получателя*

								*Пример:* 79099995522
								TXT,
								[
									'reply_markup' => [
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								$context->nextStep([static::class, 'delivery_registration_sim_input'], false);
							}); */

							// Sending the message
							$context->sendMessage(
								<<<TXT
								📞 *Пришлите номер телефона получателя*

								*Пример:* \+7 912 059 7777
								TXT,
								[
									'reply_markup' => [
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								$context->nextStep([static::class, 'delivery_registration_sim_input'], false);
							});

							break;
						case !empty($requirements['name']) && empty($account->receiver['name']):
							// Name

							/* // Sending the message
							$context->sendMessage(
								<<<TXT
								📋 *Пришлите ФИО получателя*

								*Пример:* Иван Иванов Иванович
								TXT,
								[
									'reply_markup' => [
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								$context->nextStep([static::class, 'delivery_registration_name_input'], false);
							}); */

							// Sending the message
							$context->sendMessage(
								<<<TXT
								📋 *Пожалуйста, отправьте фамилию, имя и отчество получателя*

								*Пример:* Иванов Иван Иванович
								TXT,
								[
									'reply_markup' => [
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								$context->nextStep([static::class, 'delivery_registration_name_input'], false);
							});

							break;
						case !empty($requirements['destination']) && empty($account->receiver['destination']):
							// Destination

							// Sending the message
							$context->sendMessage(
								<<<TXT
								📦 *Выберите тип доставки \(CDEK\)*
								TXT,
								[
									'reply_markup' => [
										'inline_keyboard' => [
											[
												['text' => '🏤 Пункт выдачи', 'callback_data' => 'delivery_registration_destination_office'],
												/* ['text' => '🏠 До двери', 'callback_data' => 'delivery_registration_destination_door'], */
												['text' => '🏠 Курьером до двери', 'callback_data' => 'delivery_registration_destination_door'],
											],
										],
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							);

							break;
						case !empty($requirements['address']) && empty($account->receiver['address']):
							// Address

							// Initializing title for the message
							$title = $account->receiver['destination'] === 'office' ? 'пункта выдачи' : 'получателя';

							// Sending the message
							$context->sendMessage(
								<<<TXT
								🌏 *Пришлите адрес $title*

								*Пример:* Красноярск, Ленина 24
								TXT,
								[
									'reply_markup' => [
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								$context->nextStep([static::class, 'delivery_registration_address_input'], false);
							});

							break;
						default:
							// Initializing receiver domain for the message
							$domain = static::unmarkdown($account->domain);

							// Initializing receiver SIM for the message
							$sim = static::unmarkdown((string) $account->receiver['sim'] ?? '');

							// Initializing receiver name for the message
							$name = static::unmarkdown($account->receiver['name'] ?? '');

							// Initializing receiver destination for the message
							$destination = static::unmarkdown(destination::{$account->receiver['destination'] ?? 'office'}?->label($account->language) ?? '');

							// Initializing receiver address for the message
							$address = static::unmarkdown($account->receiver['address'] ?? '');

							// Initializing the message delivery method part
							$part_method = '*Метод доставки:* *CDEK*';
							if (!empty($destination)) $part_method .= " $destination";

							// Initializing the message delivery part
							$part_delivery = '';
							if (!empty($delivery_address)) $part_delivery .= "*Адрес доставки:* $delivery_address";
							if (!empty($part_delivery)) $part_delivery = "\n$part_delivery";

							// Initializing the message receiver part
							$part_receiver = "*Получатель:* @$domain";
							if (!empty($sim)) $part_receiver .= " $sim";
							if (!empty($name)) $part_receiver .= "\n*$name*";
							if (!empty($address)) $part_receiver .= "\n\n*Адрес:* $address";

							// Sending the message
							$context->sendMessage(
								<<<TXT
								📋 *Данные для доставки*
								
								$part_method$part_delivery
								$part_receiver
								TXT
							)->then(function ($message) use ($context) {
								// Sended message

								// Sending order request
								static::order_request($context);
							});
					}
				});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Order request
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function order_request(context $context): void
	{
		// Initializing the account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			$context->getUserDataItem('cart')->then(function ($cart) use ($context) {
				// Readed the cart from the session buffer

				if (!empty($cart)) {
					// Initialized the cart

					// Initializing identifier of the cart
					$identifier = $cart->getKey();

					// Deleting the order from the user telegram buffer
					$context->deleteUserDataItem('order')
						->then(
							function ($order) use ($context, $identifier) {
								// Deleted the order from the user telegram buffer

								// Sending the message
								$context->sendMessage(
									<<<TXT
									🛒 *Корзина \#$identifier*
									TXT,
									[
										'reply_markup' => [
											'inline_keyboard' => [
												[
													['text' => '📦 Оформить заказ', 'callback_data' => 'order'],
												],
												[
													['text' => '✏️ Данные получателя', 'callback_data' => 'account_parameters_force'],
												],
												[
													['text' => '⚡ Помощь с заказом', 'url' => 'https://t.me/iarming'],
												]
											],
											'disable_notification' => true
										]
									]
								);
							}
						);
				}
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Order request
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function order(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Ending the conversation
			$context->endConversation()
				->then(function () use ($context, $account) {
					// Ended the conversation

					// Reading the order from the user telegram buffer
					$context->getUserDataItem('order')
						->then(
							function ($order) use ($context, $account) {
								// Readed the order from the user telegram buffer

								if ($order instanceof order) {
									// Initialized the order
									/*
									// Writing timer for pay into the order
									$order->term = (int) new datetime('+15 minutes')->format('U');

									if (document::update($order->__document())) {
										// Writed the order instance into the ArangoDB document */

									// Searching for the cart
									$cart = $order->cart();

									if ($cart instanceof cart) {
										// Initialized the cart

										// Skipping the order creation
										goto order;
									}
									/* } */
								}

								// Initializing the message text
								$text = $context->getCallbackQuery()?->getMessage()?->getText();

								if (!empty($text)) {
									// Initialized the message text

									// Searching for the cart identifier
									preg_match('/^.*#(\d+)$/mu', $text, $matches);

									// Initializing the cart identifier
									$identifier_cart = $matches[1] ?? null;

									// Searching for the cart
									$cart = cart::_read(
										filter: 'd._key == @_key',
										sort: 'd.updated DESC, d.created DESC, d._key DESC',
										amount: 1,
										page: 1,
										parameters: ['_key' => $identifier_cart]
									);

									if ($cart instanceof cart) {
										// Initialized the cart

										if ($cart->ordered) {
											// The cart was already ordered

											// Sending the message
											$context->sendMessage("✅ *Корзина \#$identifier_cart уже оформлена в заказ*");
										}

										// Initializing the order
										$order = $cart->order();

										if ($order) {
											// Initialized the order			

											order:

											// Writing into the telegram user buffer
											$context->setUserDataItem('order', $order)
												->then(function () use ($context, $order, $cart) {
													// Writed into the telegram user buffer			

													// Initializing identifier of the order
													$identifier_order = $order->getKey();

													if ($order->paid) {
														// The order was already paid

														// Sending the message
														$context->sendMessage("✅ *Заказ \#$identifier_order уже оплачен*");
													} else {
														// The order was nor already paid

														// Declaring the formatted list of products for message
														$list = '';

														// Declaring total cost of products
														$cost = $cart->cost($list);

														// Escaping the formatted list of products for the message
														$list = static::unmarkdown($list);

														// Initializing currency symbol
														$symbol = ($account->currency ?? currency::rub)->symbol();

														// Declaring delivery texts
														$delivery_cost = $delivery_days = $delivery_address = '';

														if (!empty($cart->buffer['delivery'])) {
															// Initialized delibery data

															// Initializing delivery cost for message
															$delivery_cost = static::unmarkdown((string) $cart->buffer['delivery']['cost']);

															// Initializing delivery days for message
															$delivery_days = static::unmarkdown((string) $cart->buffer['delivery']['days']);

															// Initializing delivery address for message
															$delivery_address = static::unmarkdown($cart->buffer['delivery']['location']['name'] . ', ' . $cart->buffer['delivery']['street']);
														}

														// Deinitializing unnecessary variables
														unset($cart);

														// Initializing the customer account
														$customer = $order->account();

														// Initializing receiver domain for the message
														$domain = static::unmarkdown($customer->domain);

														// Initializing receiver SIM for the message
														$sim = static::unmarkdown((string) $customer->receiver['sim']);

														// Initializing receiver name for the message
														$name = static::unmarkdown($customer->receiver['name']);

														// Initializing receiver address for the message
														$address = static::unmarkdown($customer->receiver['address']);

														// Initializing the message cost part
														$part_cost = "*Стоимость:* $cost$symbol";
														if (!empty($delivery_cost)) $part_cost .= " \+ $delivery_cost$symbol";
														if (!empty($delivery_days)) $part_cost .= " \($delivery_days дней\)";

														// Initializing the message delivery part
														$part_delivery = '';
														if (!empty($delivery_address)) $part_delivery .= "*Адрес доставки:* $delivery_address";
														if (!empty($part_delivery)) $part_delivery = "\n$part_delivery";

														// Initializing the message receiver part
														$part_receiver = "*Получатель:* @$domain";
														if (!empty($sim)) $part_receiver .= " $sim";
														if (!empty($name)) $part_receiver .= "\n*$name*";
														if (!empty($address)) $part_receiver .= "\n\n*Адрес:* $address";

														// Initializing the message commentary part
														$part_commentary = '';
														if (!empty($order->commentary)) $part_commentary .= "\n\n*Комментарий:* " . $order->commentary;

														// Sending the message
														$context->sendMessage(
															<<<TXT
															📦 *Заказ* \#$identifier_order

															$list
															$part_cost$part_delivery	

															$part_receiver$part_commentary
															TXT,
															[
																'reply_markup' => [
																	'inline_keyboard' => [
																		[
																			['text' => '💬 Комментарий к заказу', 'callback_data' => 'order_commentary_request']
																		],
																		[
																			['text' => '🧾 Оплатить', 'web_app' => ['url' => robokassa::link($order)]]
																		]
																	],
																	'disable_notification' => true
																]
															]
														);

														// Deinitializing unnecessary variables
														unset($cart, $list);
													}
												});
										}
									}
								}

								// Deinitializing unnecessary variables
								unset($cart);
							}
						);
				});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Account parameters
	 *
	 * Menu for request parameters for the account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function account_parameters(context $context, bool $force = false): void
	{
		// Initializing the account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			$context->getUserDataItem('cart')
				->then(
					function ($cart) use ($context, $account, $force) {
						// Readed the cart from the session buffer

						// Initializing the settings
						$settings = settings::active();

						// Initializing requirements
						$requirements = [
							'sim' => array_search('sim', $settings->input['deliveries']['chat'], true) !== false,
							'name' => array_search('name', $settings->input['deliveries']['chat'], true) !== false,
							'destination' => array_search('destination', $settings->input['deliveries']['chat'], true) !== false,
							'address' => array_search('address', $settings->input['deliveries']['chat'], true) !== false,
						];

						if (
							!$force &&
							!empty($cart) &&
							(!$requirements['sim'] or $requirements['sim'] && !empty($account->receiver['sim'])) &&
							(!$requirements['name'] or $requirements['name'] && !empty($account->receiver['name'])) &&
							(!$requirements['destination'] or $requirements['destination'] && !empty($account->receiver['destination'])) &&
							(!$requirements['address'] or $requirements['address'] && !empty($account->receiver['address']))
						) {
							// Found process for the cart offering and initialized all required account parameters

							// Sending the menu of requesting order
							static::order_request($context);
						} else {
							// Not found process for the cart offering or not initialized all required account parameters

							// Declaring the keyboard
							$keyboard = [];

							if ($requirements['sim']) {
								// Delivery receiver SIN

								// Writing into the keyboard
								$keyboard[] = [
									/* ['text' => (empty($account->receiver['sim']) ? '🔴' : '🟢') . ' SIM-номер получателя', 'callback_data' => 'receiver_sim_choose'] */
									['text' => (empty($account->receiver['sim']) ? '🔴' : '🟢') . ' Номер телефона получателя', 'callback_data' => 'receiver_sim_choose']
								];
							}

							if ($requirements['name']) {
								// Delivery receiver name

								// Writing into the keyboard
								$keyboard[] = [
									['text' => (empty($account->receiver['name']) ? '🔴' : '🟢') . ' Имя получателя', 'callback_data' => 'receiver_name_choose'],
								];
							}

							if ($requirements['destination']) {
								// Delivery receiver name

								// Writing into the keyboard
								$keyboard[] = [
									['text' => (empty($account->receiver['destination']) ? '🔴' : '🟢') . ' Тип доставки', 'callback_data' => 'receiver_destination_choose'],
								];
							}

							if ($requirements['address']) {
								// Delivery receiver address

								// Writing into the keyboard
								$keyboard[] = [
									['text' => (empty($account->receiver['address']) ? '🔴' : '🟢') . ' Адрес', 'callback_data' => 'receiver_address_choose']
								];
							}

							// Sending the message
							$context->sendMessage(
								'📑 *Настройка параметров получателя*',
								[
									'reply_markup' => [
										'inline_keyboard' => $keyboard,
										'disable_notification' => true,
									]
								]
							);
						}
					}
				);
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver SIM-number
	 *
	 * Request the receiver SIM-number and write into account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_sim_choose(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			/* // Sending the message
			$context->sendMessage(
				<<<TXT
				📞 *SIM\-номер получателя*

				Используется для генерации договора с компанией осуществляющией доставку товаров из магазина
				TXT,
				[
					'reply_markup' => [
						'keyboard' => [
							[
								['text' => '📂 Использовать номер аккаунта', 'request_contact' => true]
							],
						],
						'disable_notification' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended the message

				// Sending the message
				$context->sendMessage(
					<<<TXT
					Если SIM\-номер *отличается* от привязанного к аккаунту, то введите его вручную
					TXT,
					[
						'reply_markup' => [
							'inline_keyboard' => [
								[
									['text' => '✏️ Ввести вручную', 'callback_data' => 'receiver_sim_request']
								],
							],
							'disable_notification' => true,
						]
					]
				);
			}); */

			// Sending the message
			$context->sendMessage(
				<<<TXT
				📞 *Номер телефона получателя*

				Используется для генерации договора с компанией осуществляющией доставку товаров из магазина
				TXT,
				[
					'reply_markup' => [
						'keyboard' => [
							[
								['text' => '📂 Использовать номер аккаунта', 'request_contact' => true]
							],
						],
						'disable_notification' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended the message

				// Sending the message
				$context->sendMessage(
					<<<TXT
					Если номер телефона *отличается* от привязанного к аккаунту, то введите его вручную
					TXT,
					[
						'reply_markup' => [
							'inline_keyboard' => [
								[
									['text' => '✏️ Ввести вручную', 'callback_data' => 'receiver_sim_request']
								],
							],
							'disable_notification' => true,
						]
					]
				);
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver SIM-number request
	 *
	 * Request SIM-number
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_sim_request(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the accountnn

			/* // Sending the message
			$context->sendMessage(
				<<<TXT
				📞 *Пришлите SIM\-номер получателя*

				*Пример:* 79099995522
				TXT,
				[
					'reply_markup' => [
						'disable_notification' => true,
						'remove_keyboard' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended message

				$context->nextStep([static::class, 'receiver_sim_input'], true);
			}); */

			// Sending the message
			$context->sendMessage(
				<<<TXT
				📞 *Пришлите номер телефона получателя*

				*Пример:* \+7 912 059 7777
				TXT,
				[
					'reply_markup' => [
						'disable_notification' => true,
						'remove_keyboard' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended message

				$context->nextStep([static::class, 'receiver_sim_input'], true);
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver SIM-number from input
	 *
	 * Receive SIM-number from message
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_sim_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (SIM-number)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (SIM-number)

				// Sanitizing received text (only numbers)
				$sanitized = preg_replace('/[^\d]/', '', $text);

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (strlen($sanitized) > 6) {
						// Length of received SIM-number is more than 6 symbols

						if (strlen($sanitized) < 20) {
							// Length of received SIM-number is less than 20 symbols

							// Writing into the telegram user buffer
							$context->setUserDataItem('receiver_sim', $sanitized)
								->then(function () use ($context, $sanitized) {
									// Writed into the telegram user buffer

									/* // Sending the message
									$context->sendMessage(
										<<<TXT
										📞 *SIM\-номер:* $sanitized
										TXT,
										[
											'reply_markup' => [
												'inline_keyboard' => [
													[
														['text' => '✅ Сохранить', 'callback_data' => 'receiver_sim_write'],
														['text' => '❎ Отменить', 'callback_data' => 'receiver_sim_choose']
													]
												],
												'disable_notification' => true
											]
										]
									); */

									// Sending the message
									$context->sendMessage(
										<<<TXT
										📞 *Номер телефона:* $sanitized
										TXT,
										[
											'reply_markup' => [
												'inline_keyboard' => [
													[
														['text' => '✅ Сохранить', 'callback_data' => 'receiver_sim_write'],
														['text' => '❎ Отменить', 'callback_data' => 'receiver_sim_choose']
													]
												],
												'disable_notification' => true
											]
										]
									);

									// Resetting the buffer
									$context->endConversation();
								});
						} else {
							// Length of received SIM-number is NOT less than 20 symbols

							// Sending the message
							/* $context->sendMessage('⚠️ *SIM\-номер должен быть короче 20\-ти символов*'); */
							$context->sendMessage('⚠️ *Номер телефона должен быть короче 20\-ти символов*');
						}
					} else {
						// Length of received SIM-number is NOT more than 6 symbols

						// Sending the message
						/* $context->sendMessage('⚠️ *SIM\-номер должен быть длиннее 3\-х символов*'); */
						$context->sendMessage('⚠️ *Номер телефона должен быть длиннее 3\-х символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Delivery registration SIM-number from input
	 *
	 * Receive SIM-number from message
	 *
	 * @param context $context
	 *
	 * @return void
	 *
	 * @todo i am tired of this project
	 */
	public static function delivery_registration_sim_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (SIM-number)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (SIM-number)

				// Sanitizing received text (only numbers)
				$sanitized = preg_replace('/[^\d]/', '', $text);

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (strlen($sanitized) > 6) {
						// Length of received SIM-number is more than 6 symbols

						if (strlen($sanitized) < 20) {
							// Length of received SIM-number is less than 20 symbols

							// Writing receiver SIM-number into the account
							$account->receiver = ['sim' => (int) $sanitized] + ($account->receiver ?? []);

							// Deabstracting parameters
							if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
							if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

							if (document::update($account->__document())) {
								// Writed the account instance into the ArangoDB document

								// Abstratcting paramters
								$account->language = language::{$account->language};
								if (isset($account->currency)) $account->currency = currency::{$account->currency};

								/* // Sending the message
								$context->sendMessage(
									<<<TXT
									✅ *SIM\-номер записан:* $sanitized
									TXT,
									[
										'reply_markup' => [
											'disable_notification' => true,
											'remove_keyboard' => true
										]
									]
								)->then(function ($message) use ($context) {
									// Sended message

									// Continuing the delivery registration process 
									static::delivery_registration($context);
								}); */

								// Sending the message
								$context->sendMessage(
									<<<TXT
									✅ *Номер телефона записан:* $sanitized
									TXT,
									[
										'reply_markup' => [
											'disable_notification' => true,
											'remove_keyboard' => true
										]
									]
								)->then(function ($message) use ($context) {
									// Sended message

									// Continuing the delivery registration process 
									static::delivery_registration($context);
								});
							} else {
								// Not writed the account instance into the ArangoDB document

								// Sending the message
								/* $context->sendMessage('⚠️ *Не удалось записать SIM\-номер*'); */
								$context->sendMessage('⚠️ *Не удалось записать номер телефона*');
							}
						} else {
							// Length of received SIM-number is NOT less than 20 symbols

							// Sending the message
							/* $context->sendMessage('⚠️ *SIM\-номер должен быть короче 20\-ти символов*'); */
							$context->sendMessage('⚠️ *Номер телефона должен быть короче 20\-ти символов*');
						}
					} else {
						// Length of received SIM-number is NOT more than 6 symbols

						// Sending the message
						/* $context->sendMessage('⚠️ *SIM\-номер должен быть длиннее 3\-х символов*'); */
						$context->sendMessage('⚠️ *Номер телефона должен быть длиннее 3\-х символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver SIM-number write
	 *
	 * Write received and accepted receiver SIM-number into account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_sim_write(context $context): void
	{
		// initializing account
		$account = $context->get('account');

		if ($account) {
			// initialized the account

			// Reading from the telegram user buffer
			$context->getUserDataItem('receiver_sim')->then(function ($sim) use ($context, $account) {
				// Readed from the telegram user buffer

				if (!empty($sim)) {
					// Initialized receiver SIM-number

					// Writing receiver SIM-number into the account
					$account->receiver = ['sim' => (int) $sim] + ($account->receiver ?? []);

					// Deabstracting parameters
					if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
					if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

					if (document::update($account->__document())) {
						// Writed the account instance into the ArangoDB document

						// Abstratcting paramters
						$account->language = language::{$account->language};
						if (isset($account->currency)) $account->currency = currency::{$account->currency};

						/* // Sending the message
						$context->sendMessage(
							<<<TXT
							✅ *SIM\-номер записан:* $sim
							TXT,
							[
								'reply_markup' => [
									'disable_notification' => true,
									'remove_keyboard' => true
								]
							]
						)->then(function ($message) use ($context) {
							// Sended message

							// Deinitializing receiver SIM-number
							$context->setUserDataItem('receiver_sim', null);

							// Sending the account parameters menu
							static::account_parameters($context);
						}); */

						// Sending the message
						$context->sendMessage(
							<<<TXT
							✅ *Номер телефона записан:* $sim
							TXT,
							[
								'reply_markup' => [
									'disable_notification' => true,
									'remove_keyboard' => true
								]
							]
						)->then(function ($message) use ($context) {
							// Sended message

							// Deinitializing receiver SIM-number
							$context->setUserDataItem('receiver_sim', null);

							// Sending the account parameters menu
							static::account_parameters($context);
						});
					} else {
						// Not writed the account instance into the ArangoDB document

						// Sending the message
						/* $context->sendMessage('⚠️ *Не удалось записать SIM\-номер*'); */
						$context->sendMessage('⚠️ *Не удалось записать номер телефона*');
					}
				} else {
					// Not initialized receiver SIM-number

					// Sending the message
					/* $context->sendMessage('⚠️ *Ошибка при записи SIM\-номера*'); */
					$context->sendMessage('⚠️ *Ошибка при записи номер телефона*');
				}
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver name
	 *
	 * Request the receiver name and write into account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_name_choose(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Sending the message
			$context->sendMessage(
				<<<TXT
				📋 *ФИО получателя*

				Используется для генерации договора с компанией осуществляющией доставку товаров из магазина
				TXT,
				[
					'reply_markup' => [
						'inline_keyboard' => [
							[
								['text' => '✏️ Ввести вручную', 'callback_data' => 'receiver_name_request']
							],
						],
						'disable_notification' => true,
					]
				]
			);
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver name request
	 *
	 * Request name
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_name_request(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			/* // Sending the message
			$context->sendMessage(
				<<<TXT
				📋 *Пришлите ФИО получателя*

				*Пример:* Иван Иванов Иванович
				TXT,
				[
					'reply_markup' => [
						'disable_notification' => true,
						'remove_keyboard' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended message

				$context->nextStep([static::class, 'receiver_name_input'], true);
			}); */

			// Sending the message
			$context->sendMessage(
				<<<TXT
				📋 *Пожалуйста, отправьте фамилию, имя и отчество получателя*

				*Пример:* Иванов Иван Иванович
				TXT,
				[
					'reply_markup' => [
						'disable_notification' => true,
						'remove_keyboard' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended message

				$context->nextStep([static::class, 'receiver_name_input'], true);
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver name from input
	 *
	 * Receive name from message
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_name_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (name)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (name)

				// Sanitizing received text (only numbers)
				$sanitized = static::unmarkdown(preg_replace('/[^\s\w\-]/u', '', $text));

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (mb_strlen($sanitized) > 3) {
						// Length of received name is more than 3 symbols

						if (mb_strlen($sanitized) < 256) {
							// Length of received name is less than 256 symbols

							// Writing to the telegram user buffer
							$context->setUserDataItem('receiver_name', $sanitized)
								->then(function () use ($context, $sanitized) {

									// Sending the message
									$context->sendMessage(
										<<<TXT
										📋 *ФИО:* $sanitized
										TXT,
										[
											'reply_markup' => [
												'inline_keyboard' => [
													[
														['text' => '✅ Сохранить', 'callback_data' => 'receiver_name_write'],
														['text' => '❎ Отменить', 'callback_data' => 'receiver_name_choose']
													]
												],
												'disable_notification' => true
											]
										]
									);

									// Resetting the buffer
									$context->endConversation();
								});
						} else {
							// Length of received name is NOT less than 256 symbols

							// Sending the message
							$context->sendMessage('⚠️ *ФИО должны быть короче 256\-ти символов*');
						}
					} else {
						// Length of received name is NOT more than 3 symbols

						// Sending the message
						$context->sendMessage('⚠️ *ФИО должны быть длиннее 3\-х символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Delivery registration name from input
	 *
	 * Receive name from message
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function delivery_registration_name_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (name)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (name)

				// Sanitizing received text (only numbers)
				$sanitized = static::unmarkdown(preg_replace('/[^\s\w\-]/u', '', $text));

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (mb_strlen($sanitized) > 3) {
						// Length of received name is more than 3 symbols

						if (mb_strlen($sanitized) < 256) {
							// Length of received name is less than 256 symbols

							// Writing receiver name into the account
							$account->receiver = ['name' => $sanitized] + ($account->receiver ?? []);

							// Deabstracting parameters
							if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
							if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

							if (document::update($account->__document())) {
								// Writed the account instance into the ArangoDB document

								// Abstratcting paramters
								$account->language = language::{$account->language};
								if (isset($account->currency)) $account->currency = currency::{$account->currency};

								$context->sendMessage(
									<<<TXT
									✅ *ФИО записаны:* 
									$sanitized
									TXT,
									[
										'reply_markup' => [
											'disable_notification' => true,
											'remove_keyboard' => true
										]
									]
								)->then(function ($message) use ($context) {
									// Sended message

									// Continuing the delivery registration process 
									static::delivery_registration($context);
								});
							} else {
								// Not writed the account instance into the ArangoDB document

								// Sending the message
								$context->sendMessage('⚠️ *Не удалось записать ФИО*');
							}
						} else {
							// Length of received name is NOT less than 256 symbols

							// Sending the message
							$context->sendMessage('⚠️ *ФИО должны быть короче 256\-ти символов*');
						}
					} else {
						// Length of received name is NOT more than 3 symbols

						// Sending the message
						$context->sendMessage('⚠️ *ФИО должны быть длиннее 3\-х символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver name write
	 *
	 * Write received and accepted receiver name into account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_name_write(context $context): void
	{
		// initializing account
		$account = $context->get('account');

		if ($account) {
			// initialized the account

			// Reading from the telegram user buffer
			$context->getUserDataItem('receiver_name')
				->then(function ($name) use ($context, $account) {
					// Readed from the telegram user buffer

					if (!empty($name)) {
						// Initialized receiver name

						// Writing receiver name into the account
						$account->receiver = ['name' => $name] + ($account->receiver ?? []);

						// Deabstracting parameters
						if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
						if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

						if (document::update($account->__document())) {
							// Writed the account instance into the ArangoDB document

							// Abstratcting paramters
							$account->language = language::{$account->language};
							if (isset($account->currency)) $account->currency = currency::{$account->currency};

							// Sending the message
							$context->sendMessage(
								<<<TXT
								✅ *ФИО записаны:* 
								$name
								TXT,
								[
									'reply_markup' => [
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								// Deinitializing receiver name
								$context->setUserDataItem('receiver_name', null);

								// Sending the account parameters menu
								static::account_parameters($context);
							});
						} else {
							// Not writed the account instance into the ArangoDB document

							// Sending the message
							$context->sendMessage('⚠️ *Не удалось записать ФИО*');
						}
					} else {
						// Not initialized receiver name

						// Sending the message
						$context->sendMessage('⚠️ *Ошибка при регистрации ФИО*');
					}
				});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}


	/**
	 * Receiver destination
	 *
	 * Request the receiver destination and write into account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_destination_choose(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Sending the message
			$context->sendMessage(
				<<<TXT
				📦 *Выберите тип доставки \(CDEK\)*

				Используется для генерации договора с компанией осуществляющией доставку товаров из магазина
				TXT,
				[
					'reply_markup' => [
						'inline_keyboard' => [
							[
								['text' => '🏤 Пункт выдачи', 'callback_data' => 'receiver_destination_office'],
								['text' => '🏠 До двери', 'callback_data' => 'receiver_destination_door'],
							],
						],
						'disable_notification' => true,
					]
				]
			);
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver destination office
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_destination_office(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing the destination
			$destination = destination::office;

			if (!empty($destination)) {
				// Initialized receiver destination

				// Writing receiver destination into the account
				$account->receiver = ['destination' => $destination->name] + ($account->receiver ?? []);

				// Initializing destination for the message
				$type = $destination->label($account->language);

				// Deabstracting parameters
				if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
				if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

				if (document::update($account->__document())) {
					// Writed the account instance into the ArangoDB document

					// Abstratcting paramters
					$account->language = language::{$account->language};
					if (isset($account->currency)) $account->currency = currency::{$account->currency};

					// Sending the message
					$context->sendMessage(
						<<<TXT
						✅ *Тип доставки записан:* $type
						TXT,
						[
							'reply_markup' => [
								'disable_notification' => true,
								'remove_keyboard' => true
							]
						]
					)->then(function ($message) use ($context) {
						// Sended message

						// Sending the account parameters menu
						static::account_parameters($context);
					});
				} else {
					// Not writed the account instance into the ArangoDB document

					// Sending the message
					$context->sendMessage('⚠️ *Не удалось записать тип доставки*');
				}
			} else {
				// Not initialized receiver name

				// Sending the message
				$context->sendMessage('⚠️ *Ошибка при регистрации типа доставки*');
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Delivery registration destination office
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function delivery_registration_destination_office(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing the destination
			$destination = destination::office;

			if (!empty($destination)) {
				// Initialized receiver destination

				// Writing receiver destination into the account
				$account->receiver = ['destination' => $destination->name] + ($account->receiver ?? []);

				// Initializing destination for the message
				$type = $destination->label($account->language);

				// Deabstracting parameters
				if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
				if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

				if (document::update($account->__document())) {
					// Writed the account instance into the ArangoDB document

					// Abstratcting paramters
					$account->language = language::{$account->language};
					if (isset($account->currency)) $account->currency = currency::{$account->currency};

					// Sending the message
					$context->sendMessage(
						<<<TXT
						✅ *Тип доставки записан:* $type
						TXT,
						[
							'reply_markup' => [
								'disable_notification' => true,
								'remove_keyboard' => true
							]
						]
					)->then(function ($message) use ($context) {
						// Sended message

						// Continuing the delivery registration process
						static::delivery_registration($context);
					});
				} else {
					// Not writed the account instance into the ArangoDB document

					// Sending the message
					$context->sendMessage('⚠️ *Не удалось записать тип доставки*');
				}
			} else {
				// Not initialized receiver name

				// Sending the message
				$context->sendMessage('⚠️ *Ошибка при регистрации типа доставки*');
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver destination door
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_destination_door(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing the destination
			$destination = destination::door;

			if (!empty($destination)) {
				// Initialized receiver destination

				// Writing receiver destination into the account
				$account->receiver = ['destination' => $destination->name] + ($account->receiver ?? []);

				// Initializing destination for the message
				$type = $destination->label($account->language);

				// Deabstracting parameters
				if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
				if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

				if (document::update($account->__document())) {
					// Writed the account instance into the ArangoDB document

					// Abstratcting paramters
					$account->language = language::{$account->language};
					if (isset($account->currency)) $account->currency = currency::{$account->currency};

					// Sending the message
					$context->sendMessage(
						<<<TXT
						✅ *Тип доставки записан:* $type
						TXT,
						[
							'reply_markup' => [
								'disable_notification' => true,
								'remove_keyboard' => true
							]
						]
					)->then(function ($message) use ($context) {
						// Sended message

						// Sending the account parameters menu
						static::account_parameters($context);
					});
				} else {
					// Not writed the account instance into the ArangoDB document

					// Sending the message
					$context->sendMessage('⚠️ *Не удалось записать тип доставки*');
				}
			} else {
				// Not initialized receiver name

				// Sending the message
				$context->sendMessage('⚠️ *Ошибка при регистрации типа доставки*');
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Delivery registration destination door
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function delivery_registration_destination_door(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing the destination
			$destination = destination::door;

			if (!empty($destination)) {
				// Initialized receiver destination

				// Writing receiver destination into the account
				$account->receiver = ['destination' => $destination->name] + ($account->receiver ?? []);

				// Initializing destination for the message
				$type = $destination->label($account->language);

				// Deabstracting parameters
				if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
				if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

				if (document::update($account->__document())) {
					// Writed the account instance into the ArangoDB document

					// Abstratcting paramters
					$account->language = language::{$account->language};
					if (isset($account->currency)) $account->currency = currency::{$account->currency};

					// Sending the message
					$context->sendMessage(
						<<<TXT
						✅ *Тип доставки записан:* $type
						TXT,
						[
							'reply_markup' => [
								'disable_notification' => true,
								'remove_keyboard' => true
							]
						]
					)->then(function ($message) use ($context) {
						// Sended message

						// Continuing the delivery registration process
						static::delivery_registration($context);
					});
				} else {
					// Not writed the account instance into the ArangoDB document

					// Sending the message
					$context->sendMessage('⚠️ *Не удалось записать тип доставки*');
				}
			} else {
				// Not initialized receiver name

				// Sending the message
				$context->sendMessage('⚠️ *Ошибка при регистрации типа доставки*');
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver address
	 *
	 * Request the receiver address and write into account
	 *
	 * @param context $context
	 * @param bool $office Is the destination are office?
	 *
	 * @return void
	 */
	public static function receiver_address_choose(context $context, bool $office = false): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing title for the message
			$title = $office ? 'пункта выдачи' : 'получателя';

			// Sending the message
			$context->sendMessage(
				<<<TXT
				🌏 *Адрес $title*

				Используется для генерации договора с компанией осуществляющией доставку товаров из магазина
				TXT,
				[
					'reply_markup' => [
						'inline_keyboard' => [
							[
								['text' => '✏️ Ввести вручную', 'callback_data' => 'receiver_address_request_' . ($office ? 'office' : 'door')]
							],
						],
						'disable_notification' => true,
					]
				]
			);
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver address request
	 *
	 * Request address
	 *
	 * @param context $context
	 * @param bool $office Is the destination are office?
	 *
	 * @return void
	 */
	public static function receiver_address_request(context $context, bool $office = false): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing title for the message
			$title = $office ? 'пункта выдачи' : 'получателя';

			// Sending the message
			$context->sendMessage(
				<<<TXT
				🌏 *Пришлите адрес $title*

				*Пример:* Красноярск, Ленина 24
				TXT,
				[
					'reply_markup' => [
						'disable_notification' => true,
						'remove_keyboard' => true
					]
				]
			)->then(function ($message) use ($context) {
				// Sended message

				$context->nextStep([static::class, 'receiver_address_input'], true);
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver address from input
	 *
	 * Receive address from message
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_address_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (address)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (address)

				// Sanitizing received text
				$sanitized = static::unmarkdown($text);

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (mb_strlen($sanitized) > 6) {
						// Length of received address is more than 6 symbols

						if (mb_strlen($sanitized) < 256) {
							// Length of received address is less than 256 symbols

							// Writing to the telegram user buffer
							$context->setUserDataItem('receiver_address', $sanitized)
								->then(function () use ($context, $sanitized) {
									// Writed to the telegram user buffer

									$context->sendMessage(
										<<<TXT
										🌏 *Адрес:* $sanitized
										TXT,
										[
											'reply_markup' => [
												'inline_keyboard' => [
													[
														['text' => '✅ Сохранить', 'callback_data' => 'receiver_address_write'],
														['text' => '❎ Отменить', 'callback_data' => 'receiver_address_choose']
													]
												],
												'disable_notification' => true
											]
										]
									);

									// Resetting the buffer
									$context->endConversation();
								});
						} else {
							// Length of received address is NOT less than 256 symbols

							// Sending the message
							$context->sendMessage('⚠️ *Адрес должен быть короче 256\-ти символов*');
						}
					} else {
						// Length of received address is NOT more than 6 symbols

						// Sending the message
						$context->sendMessage('⚠️ *Адрес должен быть длиннее 6\-ти символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Delivery registration address from input
	 *
	 * Receive address from message
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function delivery_registration_address_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (address)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (address)

				// Sanitizing received text
				$sanitized = static::unmarkdown($text);

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (mb_strlen($sanitized) > 6) {
						// Length of received address is more than 6 symbols

						if (mb_strlen($sanitized) < 256) {
							// Length of received address is less than 256 symbols

							// Writing receiver address into the account
							$account->receiver = ['address' => $sanitized] + ($account->receiver ?? []);

							// Deabstracting parameters
							if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
							if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

							if (document::update($account->__document())) {
								// Writed the account instance into the ArangoDB document

								// Abstratcting paramters
								$account->language = language::{$account->language};
								if (isset($account->currency)) $account->currency = currency::{$account->currency};

								// Sending the message
								$context->sendMessage(
									<<<TXT
									✅ *Адрес записан:* 
									$sanitized
									TXT,
									[
										'reply_markup' => [
											'disable_notification' => true,
											'remove_keyboard' => true
										]
									]
								)->then(function ($message) use ($context) {
									// Sended the message

									// Continuing the delivery registration process
									static::delivery_registration($context);
								});
							} else {
								// Not writed the account instance into the ArangoDB document

								// Sending the message
								$context->sendMessage('⚠️ *Не удалось записать адрес*');
							}
						} else {
							// Length of received address is NOT less than 256 symbols

							// Sending the message
							$context->sendMessage('⚠️ *Адрес должен быть короче 256\-ти символов*');
						}
					} else {
						// Length of received address is NOT more than 6 symbols

						// Sending the message
						$context->sendMessage('⚠️ *Адрес должен быть длиннее 6\-ти символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Receiver address write
	 *
	 * Write received and accepted receiver address into account
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function receiver_address_write(context $context): void
	{
		// initializing account
		$account = $context->get('account');

		if ($account) {
			// initialized the account

			$context->getUserDataItem('receiver_address')->then(function ($address) use ($context, $account) {
				// Initialized receiver address

				if (!empty($address)) {
					// Initialized receiver address

					// Writing receiver address into the account
					$account->receiver = ['address' => $address] + ($account->receiver ?? []);

					// Deabstracting parameters
					if ($account->language instanceof language) $account->language = $account->language?->name ?? settings::active()?->language->name;
					if (isset($account->currency) && $account->currency instanceof currency) $account->currency = $account->currency?->name ?? settings::active()?->currency->name;

					if (document::update($account->__document())) {
						// Writed the account instance into the ArangoDB document

						// Abstratcting paramters
						$account->language = language::{$account->language};
						if (isset($account->currency)) $account->currency = currency::{$account->currency};

						// Sending the message
						$context->sendMessage(
							<<<TXT
							✅ *Адрес записан:* 
							$address
							TXT,
							[
								'reply_markup' => [
									'disable_notification' => true,
									'remove_keyboard' => true
								]
							]
						)->then(function ($message) use ($context) {
							// Sended the message

							// Deinitializing receiver address
							$context->setUserDataItem('receiver_address', null);

							// Sending the account parameters menu
							static::account_parameters($context);
						});
					} else {
						// Not writed the account instance into the ArangoDB document

						// Sending the message
						$context->sendMessage('⚠️ *Не удалось записать адрес*');
					}
				} else {
					// Not initialized receiver address

					// Sending the message
					$context->sendMessage('⚠️ *Ошибка при записи адреса*');
				}
			});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Order commentary request
	 *
	 * Request commentary
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function order_commentary_request(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Reading from the telegram user buffer
			$context->getUserDataItem('order')
				->then(function ($order) use ($context) {
					// Readed from the telegram user buffer

					if ($order instanceof order) {
						// Initialized the order

						// Initializing the order identifier for the message
						$identifier = $order->getKey();

						// Sending the message
						$context->sendMessage(
							<<<TXT
							💬 *Пришлите комментарий к заказу \#$identifier*

							Текст для продавца в свободной форме
							TXT,
							[
								'reply_markup' => [
									'inline_keyboard' => [
										[
											['text' => '❎ Отменить', 'callback_data' => 'order']
										],
									],
									'disable_notification' => true,
									'remove_keyboard' => true
								]
							]
						)->then(function ($message) use ($context) {
							// Sended message

							$context->nextStep([static::class, 'order_commenary_input'], true);
						});
					}
				});
		}

		// Deinitializing unnecessary variables
		unset($account);
	}

	/**
	 * Order commentary from input
	 *
	 * Receive order commentary from message
	 *
	 * @param context $context
	 *
	 * @return void
	 */
	public static function order_commenary_input(context $context): void
	{
		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			// Initializing received text (address)
			$text = $context->getMessage()?->getText();

			if (!empty($text)) {
				// Initialized received text (address)

				// Sanitizing received text
				$sanitized = static::unmarkdown($text);

				if (!empty($sanitized)) {
					// Initialized sanitized received text (only numbers)

					if (mb_strlen($sanitized) > 3) {
						// Length of received address is more than 3 symbols

						if (mb_strlen($sanitized) < 256) {
							// Length of received address is less than 256 symbols

							// Readed the order from the user telegram buffer
							$context->getUserDataItem('order')
								->then(
									function ($order) use ($context, $account, $sanitized) {
										// Readed the order from the user telegram buffer

										if ($order instanceof order) {
											// Initialized the order

											// Writing the commentary into the order
											$order->commentary = $sanitized;

											if (document::update($order->__document())) {
												// Writed the order instance into the ArangoDB document

												// Sending the message
												$context->sendMessage(
													<<<TXT
													✅ *Комментарий записан:* 
													$sanitized
													TXT,
													[
														'reply_markup' => [
															'disable_notification' => true,
															'remove_keyboard' => true
														]
													]
												)->then(function ($message) use ($context) {
													// Sended the message

													// Continuing the delivery registration process
													static::order($context);
												});
											}
										}
									}
								);
						} else {
							// Length of received address is NOT less than 256 symbols

							// Sending the message
							$context->sendMessage('⚠️ *Адрес должен быть короче 256\-ти символов*');
						}
					} else {
						// Length of received address is NOT more than 3 symbols

						// Sending the message
						$context->sendMessage('⚠️ *Адрес должен быть длиннее 3\-ти символов*');
					}
				}
			}
		}

		// Deinitializing unnecessary variables
		unset($account);
	}
}
